import json
import logging
import traceback
import zlib
from locust import User, events
import time
import websocket
from ..common import logger
from . import api


class WebSocket(websocket.WebSocket):
    is_locust = False
    is_distinguish_title = False

    def request_success(self, name, response_time: float = 0, request_type: str = "websockt"):
        if self.is_distinguish_title is True:
            name = self.kwargs[1] + '_' + name
        events.request_success.fire(request_type=request_type, name=name, response_time=response_time,
                                    response_length=0)

    def request_failure(self, name, exception, response_time: float = 0, request_type: str = "websockt"):
        if self.is_distinguish_title is True:
            name = self.kwargs[1] + '_' + name
        events.request_failure.fire(request_type=request_type, name=name, response_time=response_time,
                                    response_length=0, exception=exception)

    def connect(self, host, name, timeout=1, header=None):
        self.kwargs = host, name, timeout, header
        if header is None:
            header = {}
        start_time = time.time()
        try:
            self.conn = super().connect(host, header=header, verify_ssl=False, skip_negotiation=False)
            self.settimeout(timeout)
        except Exception as e:
            total_time = int((time.time() - start_time) * 1000)
            self.request_failure("连接", e, response_time=total_time)
            logger.error(traceback.format_exc())
            raise e
        else:
            total_time = int((time.time() - start_time) * 1000)
            logger.debug("websocket建联:url:{},header:{}，用时:{}毫秒".format(host, header, total_time))
            self.request_success("连接", response_time=total_time)

    def reconnect(self):
        self.connect(*self.kwargs)

    def recv_data(self) -> (int, dict):
        """
        获取服务端下发内容， 阻塞形式的，如想不阻塞，请在connect的时候设置超时时间
        :return: 类型，下发内容
        """
        _type, data = super().recv_data()
        if _type == 2:
            data = json.loads(zlib.decompress(data, 16 + zlib.MAX_WBITS))
        else:
            data = json.loads(data)
        logger.debug('---recv---:type:{},{}'.format(_type, data))
        if 'ping' in data:
            self.request_success("ping接收")
        elif 'pong' in data:
            self.request_success("pong接收")
        else:
            self.request_success("其他接收")
        return _type, data

    def send_by_list(self, data: list) -> None:
        """
        发送消息列表
        :param data:
        :return:
        """
        for i in data:
            self.send(i)

    def send(self, msg: dict):
        """
        发送消息
        :param msg:消息
        :return:
        """
        msg = json.dumps(msg)
        start_time = time.time()
        logger.debug('---send---:{}'.format(msg))
        super().send(msg)
        self.request_success("发送消息", response_time=time.time() - start_time)

    def ping(self):
        """
        发送ping请求
        :return:
        """
        msg = {"ping": int(time.time() * 1000)}
        msg = json.dumps(msg)
        logger.debug('---ping---:{}'.format(msg))
        start_time = time.time()
        super().send(msg)
        self.request_success("心跳ping日志", response_time=time.time()-start_time)

    def pong(self):
        """
        发送pong请求
        :return:
        """
        msg = {"pong": int(time.time() * 1000)}
        msg = json.dumps(msg)
        start_time = time.time()
        super().send(msg)
        logger.debug('发送pong请求:{}'.format(msg))
        self.request_success("心跳pong日志", response_time=time.time()-start_time)

    def get_status(self):
        """
        获取状态
        :return:
        """
        super().getstatus()

    def set_status(self, status):
        super().handshake_response.status = status





class webSocket(api.Api):

    def __init__(self, kwargs):
        super().__init__()
        self.kwargs = kwargs

    def get_client(self, api_id, *topics, header=None, **kwargs) -> websocket.WebSocketApp:
        api_data = self.get_api(api_id)
        path = api_data['path']
        url = '{}://{}{}'.format(self.kwargs.get('agreement', 'ws'), self.kwargs['host'], path)
        on_open = kwargs.get('on_open', self.on_open)
        on_error = kwargs.get('on_error', self.on_error)
        on_close = kwargs.get('on_open', self.on_close)
        on_pong = kwargs.get('on_pong', self.on_pong)
        on_ping = kwargs.get('on_ping', self.on_ping)
        on_message = kwargs.get('on_message', self.on_message)
        on_data = kwargs.get('on_data', self.on_data)
        on_cont_message = kwargs.get('on_cont_message')
        ws = websocket.WebSocketApp(url=url, header=header, on_open=on_open, on_error=on_error, on_close=on_close,
                                    on_pong=on_pong, on_ping=on_ping, on_message=on_message)
        ws.topics = topics
        ws.title = api_data['title']
        return ws

    def on_data(self, ws:websocket.WebSocketApp, message, data_type, flag):
        logger.debug("{}内容：{}，类型：{}，flage：{}".format(ws.title, message, data_type, flag))

    def on_close(self, ws:websocket.WebSocketApp, close_status_code, close_msg):
        logger.debug("{}关闭连接,状态码:{}, 关闭消息:{}.....".format(ws.title, close_status_code, close_msg))

    def on_error(self, ws:websocket.WebSocketApp, exception):
        if getattr(a, 'format_exc', None) is not None:
            logger.debug("异常():{}".format(ws.title, exception.format_exc()))
        else:
            logger.exception("异常({})".format(ws.title))

    def on_open(self, ws:websocket.WebSocketApp):
        logger.debug("websocket()连接成功.....".format(ws.title))
        ws.open_start_time = time.time()
        for topic in ws.topics:
            ws.send(json.dumps(topic))

    def on_pong(self, ws:websocket.WebSocketApp, message):
        logger.debug("{}收到pong消息:{}".format(ws.title, message))

    def on_ping(self, ws:websocket.WebSocketApp, message):
        ws.send(json.dumps({"pong": int(time.time() * 1000)}), websocket.ABNF.OPCODE_PONG)
        logger.debug("{}收到ping消息:{}".format(ws.title, message))

    def on_message(self, ws:websocket.WebSocketApp, message):
        logger.debug("{}收到消息:{}".format(ws.title, message))



import copy
import datetime
import json
import os
import sys
import queue
import shutil
import traceback
import uuid
import subprocess
from functools import cmp_to_key
import urllib3

urllib3.disable_warnings()
import pytest
import gevent
import time
from multiprocessing import Process
from locust.stats import stats_printer, stats_history
from locust.env import Environment
import locust, logging
from locust.log import greenlet_exception_logger
from locust.user.inspectuser import print_task_ratio
from .common import constant, config, common
from .tool import yapi
from autoTestScheme.configuration import robot
from .common.logger import logger
from . import conf

json_name = 'autoTestScheme'


def run_case(data: list, _uuid: str, env: str):
    if type(data) == dict:
        # 为适配无分组用例
        data = [data]
    _data = {}
    _data['__env'] = env
    num = 1
    for i in data:
        __file = i.get('__file')
        __func = i.get('__func')
        if __file not in list(_data.keys()):
            _data[__file] = {}
        if __func not in list(_data[__file].keys()):
            _data[__file][__func] = []
        i['__order'] = num
        _data[__file][__func].append(i)
        num += 1
    folder = os.path.join(constant.RUN_TMP_DIR, _uuid)
    if not os.path.exists(folder):
        os.makedirs(folder)
    _path = os.path.join(folder, 'case.json')
    logger.info('执行数据：{}, 数据地址:{}'.format(_data, _path))
    with open(_path, 'w+')as f:
        f.write(json.dumps(_data, sort_keys=True, indent=4, separators=(',', ':')))

    _path = os.path.join(constant.TMP_REPORTS_DIR, _uuid)
    command = ['-s', '--capture=no', '-v', '--alluredir', _path, '--clean-alluredir',
               '--cache-clear', '-W', 'ignore:Module already imported:pytest.PytestWarning', '--{}'.format(json_name),
               _uuid]
    logger.info('执行命令:{}'.format(command))
    pytest.main(command)


class PropogateHandler(logging.Handler):
    def emit(self, record: logging.LogRecord) -> None:
        logging.getLogger(record.name).handle(record)


@common.singleton
class Run(object):

    def __init__(self):
        self.all_data = []
        self.all_data_by_channel_id = {}
        self.uuid_list = []
        self.data_folder = None
        self.api_data = []
        self.result_folder = None

    def register_tags(self, tags: dict):
        """
        注册标签列表
        :param tags:
        :return:
        """
        for key, value in tags.items():
            self.register_tag(key, value)

    def set_logger(self, level="DEBUG", log_file_level="DEBUG", is_allure=True, logger_folder_name='logs',
                   logger_start_str='run'):
        """
        修改终端日志登记
        :param _str:
        :return:
        """
        logger.remove()
        logger.add(sys.stderr, level=level)
        log_folder = os.path.join(constant.BASE_DIR, logger_folder_name)
        if not os.path.exists(log_folder):
            os.makedirs(log_folder)
        log_path = os.path.join(log_folder, logger_start_str + '_{time}.log')
        logger.add(log_path, level=log_file_level, rotation="200 MB")
        if is_allure is True:
            logger.add(PropogateHandler(), level=level, format="{time:YYYY-MM-DD at HH:mm:ss} | {message}")

    def load_case(self, folder_name):
        """
        加载用例数据
        :param folder_name: 启动脚本同级目录名
        :return:
        """
        self.data_folder = os.path.join(constant.BASE_DIR, folder_name)

    def load_allure_tmp(self, folder_name):
        """
        定义allure缓存目录
        :param folder_name: 启动脚本同级目录名
        :return:
        """
        constant.TMP_REPORT_DIR = os.path.join(constant.BASE_DIR, folder_name)

    def append_locust_task(self, *args, **kwargs):
        gevent.spawn(*args, **kwargs)

    def run_locust(self, case: list, env: str, tag: set, host: str, ui_host: str = '0.0.0.0', ui_port: int = 8089):
        env = Environment(
            locustfile='',
            user_classes=case,
            tags=tag,
            events=locust.events,
            host=host
        )
        print_task_ratio(case)
        runner = env.create_local_runner()
        # main_greenlet = runner.greenlet
        web_ui = env.create_web_ui(ui_host, ui_port)
        env.events.init.fire(environment=env, runner=runner, web_ui=web_ui)
        main_greenlet = web_ui.greenlet
        stats_printer_greenlet = gevent.spawn(stats_printer(runner.stats))
        # setup_logging(loglevel, logfile)
        logger = logging.getLogger(__name__)
        greenlet_exception_handler = greenlet_exception_logger(logger)
        stats_printer_greenlet.link_exception(greenlet_exception_handler)
        gevent.spawn(stats_history, runner)
        main_greenlet.join()

    def run(self):
        self.init_dir()
        tag_name = ';'.join([conf.settings.run.tag_env_list[i] for i in conf.settings.run.test_tags])
        logger.info('开始使用 {} 个进程 {} {}(标签:{})的 {} 用例'.format(conf.settings.run.process_num,
                                                       '运行' if conf.settings.run.is_debug is False else '调试'
                                                        , conf.settings.get_env_name(), tag_name,
                                                       conf.settings.run.test_case))
        logger.info('开始收集测试用例。。。。。')
        group, surplus_data = self.collect_use_cases()

        group_length = len(group)
        surplus_length = len(surplus_data)

        logger.info('分组用例：{}'.format(group))
        logger.info('未分组用例：{}'.format(surplus_data))

        # 执行分组用例
        if len(group) == 0:
            logger.info('分组用例为空')
            del group
        elif len(group) == 1:
            self.run_case(group[0])
        else:
            self.start_process(self.list_transformation_queue(group))

        # 执行未分组用例
        if len(surplus_data) == 0:
            logger.info('未分组用例为空')
        elif (len(surplus_data) == 1 and len(
                [j for i in surplus_data for j in i]) == 1) or conf.settings.run.process_num == 1:
            for i in surplus_data:
                self.run_case(i)
        else:
            for surplus in surplus_data:
                self.start_process(self.list_transformation_queue(surplus))
        if group_length == 0 and surplus_length == 0:
            logger.info("结束测试。。。。")
            return
        self.collect_reports()
        self.update_environment(constant.TMP_REPORT_DIR)
        logger.error('运行完成....')

    def run_case(self, data):
        _uuid = str(uuid.uuid4())
        self.uuid_list.append(_uuid)
        run_case(data, _uuid, conf.settings.current_env)

    def start_process(self, group_queue):
        total = group_queue.qsize()
        pro_list = []
        import threading
        while len(pro_list) != 0 or group_queue.qsize() != 0:
            for pro in pro_list:
                if pro.is_alive() is False:
                # if pro.isAlive() is False:
                    name = pro.name
                    pro_list.remove(pro)
            if len(pro_list) < conf.settings.run.process_num and group_queue.qsize() > 0:
                try:
                    data = group_queue.get_nowait()
                    _uuid = str(uuid.uuid4())
                    self.uuid_list.append(_uuid)
                    logger.info("执行进度:{}/{}".format(group_queue.qsize(), total))
                    p = threading.Thread(target=run_case, args=(data, _uuid, conf.settings.current_env))
                    p.setName(_uuid)
                    p.start()

                    # p = Process(target=run_case, args=(data, _uuid, conf.settings.current_env))
                    # p.start()
                    # p.name = _uuid
                    pro_list.append(p)
                except Exception:
                    logger.error(traceback.format_exc())
            time.sleep(2)

    def list_transformation_queue(self, _list) -> queue.Queue:
        _result = queue.Queue()
        for i in range(len(_list)):
            _result.put(_list[len(_list) - 1 - i])
        del _list
        return _result

    def collect_use_cases(self):
        group_data = []
        self.init_all_data()
        # 区分下渠道
        _all_data = {}
        for i in self.all_data:
            if i.get('tag') not in list(_all_data.keys()):
                _all_data[i.get('tag')] = []
            _all_data[i.get('tag')].append(i)

        for channel_data in list(_all_data.values()):
            channel_group_data = self.get_group_by_channel(copy.deepcopy(channel_data))
            group_data += channel_group_data

        surplus_data = self.get_surplus_data(copy.deepcopy(_all_data), copy.deepcopy(group_data))
        return group_data, surplus_data

    def get_group_by_channel(self, channel_data: list) -> list:
        """
        获取渠道的分组
        :param channel_data: 渠道数据
        :return:
        """
        group = []
        # dependent_list
        dependent_list = []
        for data in channel_data:
            if self.check_data(data):
                continue
            if data.get('dependent') is not None:
                dependent_list.append(data['dependent'])
        for data in channel_data:
            if self.check_data(data):
                continue
            if data.get('dependent') is not None and data.get('id') not in dependent_list:
                _data_list = self.get_dependent_data(data)
                for _data in _data_list:
                    _data.reverse()
                    group.append(_data)
        return group

    def check_data(self, data, is_file=True):
        # 调试时过滤
        if conf.settings.run.is_debug is True and 'debug' not in data.get('tags'):
            return True
        if data.get('__file') != conf.settings.current_env and conf.settings.run.test_case != 'all' and is_file is True:
            return True
        return False

    def get_surplus_data(self, all_data: dict, channel_group_data: list):
        surplus = {}
        for channel_data in list(all_data.values()):
            for data in channel_data:
                __file = data.get('__file')
                __func = data.get('__func')
                __dependent_class = data.get('__dependent_class')
                if self.check_data(data):
                    continue
                if data not in [j for i in channel_group_data for j in i]:
                    if __file not in list(surplus.keys()):
                        surplus[__file] = {}
                    if __func not in list(surplus[__file].keys()):
                        surplus[__file][__func] = []
                        surplus[__file]['__dependent_class'] = __dependent_class
                    surplus[__file][__func].append(data)
        surplus_data = list(surplus.items())
        sorted(surplus_data, key=cmp_to_key(self.compare))
        surplus_result = []
        for i in surplus_data:
            if '__dependent_class' in list(i[1].keys()): del i[1]['__dependent_class']
            new_i = list(i[1].items())
            new_i.sort()
            surplus_result += [j[1] for j in new_i]
        return surplus_result

    def collect_reports(self):
        for root, dirs, files in os.walk(constant.TMP_REPORTS_DIR):
            if os.path.basename(root) in self.uuid_list:
                for name in files:
                    _path = os.path.join(root, name)
                    if_copy = True
                    if name.endswith('result.json'):
                        _tmp = config.Json(_path)
                        # 过滤无数据用例
                        filter = "@pytest.mark.skip(reason='got empty parameter set ['data']"
                        labels = _tmp.get_key('labels')
                        for label in labels:
                            if label.get('name') == 'tag':
                                if label.get('value').startswith(filter):
                                    if_copy = False
                                    break
                        if if_copy is True:
                            # 删除防止参数显示太长导致用例标题显示过大
                            _tmp.put_key('parameters', [])
                            # 删除防止数据被当成重试用例
                            _tmp.remove_key('historyId')
                    if if_copy is True:
                        shutil.copyfile(_path, os.path.join(constant.TMP_REPORT_DIR, name))
        time.sleep(10)

    def compare(self, data1, data2):
        if data1[1].get('__dependent_class') == data2[0]:
            return 1
        return 0

    def init_all_data(self):
        """
        获取所有用例数据
        :return:
        """
        logger.info(self.data_folder)
        for root, dirs, files in os.walk(self.data_folder):
            for name in files:
                _path = os.path.join(root, name)
                if name.startswith('test_') is True and name.endswith('.json') is True:
                    _data = config.Json(_path).get_object()
                    if type(_data) != list or len(_data) == 0 or _data[0].get('__file') is None:
                        continue
                    __init = _data[0]
                    for i in _data[1:]:
                        case = copy.deepcopy(__init)
                        case.update(i)
                        for tag in list(set(conf.settings.run.test_tags).intersection(set(case.get('tags', [])))):
                            case['tag'] = tag
                            for subTag in case.pop('subTags', ['default']):
                                if case.get('id') is None:
                                    case['id'] = str(uuid.uuid4())
                                case['subTag'] = subTag
                                for sonTag in case.pop('sonTags', ['default']):
                                    case['sonTag'] = sonTag
                                    if tag not in list(self.all_data_by_channel_id.keys()):
                                        self.all_data_by_channel_id[tag] = {}
                                    if case['id'] not in self.all_data_by_channel_id[tag]:
                                        self.all_data_by_channel_id[tag][case['id']] = []
                                    self.all_data_by_channel_id[tag][case['id']].append(copy.deepcopy(case))
                                    self.all_data.append(copy.deepcopy(case))
        self.replace_base_data()

    def get_dependent_data(self, data):
        result = []
        dependent = data.get('dependent')
        channel = data.get('tag')
        if channel in list(self.all_data_by_channel_id.keys()) and \
                dependent in list(self.all_data_by_channel_id[channel].keys()):
            _dep_list = self.all_data_by_channel_id[channel].get(dependent)
            for _dep in _dep_list:
                if _dep.get('dependent') is not None:
                    result += [[data] + i for i in self.get_dependent_data(_dep)]
                else:
                    result.append([data, _dep])
        return result

    def replace_base_data(self):
        """
        替换继承数据
        :return:
        """
        _num = 0
        for i in range(len(self.all_data)):
            data = self.all_data[i]
            _base = data.get('__base')
            if _base is None:
                continue

            # 使base可继承别的渠道数据，继承顺序：优先本渠道-->其次其他渠道
            _base_data = {}
            for v in list(self.all_data_by_channel_id.values()):
                for k, j in v.items():
                    if k not in list(_base_data.keys()):
                        _base_data[k] = j
            channel = data.get('tag')
            _id = data.get('id')
            if channel in list(self.all_data_by_channel_id.keys()) and _base in list(
                    self.all_data_by_channel_id[channel].keys()):
                data = self.merge_dict(copy.deepcopy(self.all_data_by_channel_id[channel][_base]), copy.deepcopy(data))
                del data['__base']
                self.all_data[i] = data
                self.all_data_by_channel_id[data['tag']][data.get('id')] = data
            elif _base in list(_base_data.keys()):
                data = self.merge_dict(copy.deepcopy(_base_data[_base]), copy.deepcopy(data))
                del data['__base']
                self.all_data[i] = data
                self.all_data_by_channel_id[data['tag']][data.get('id')] = data
            elif _base is not None:
                logger.info("数据对应base不存在:{}".format(_base))
                self.all_data[i] = {}
                _num += 1
                self.all_data_by_channel_id[data['tag']][data.get('id')] = data

        for i in range(_num):
            self.all_data.remove({})

    def merge_dict(self, _json1: dict, _json2: dict):
        for k in list(_json2.keys()):
            v = _json2[k]
            if type(v) != dict:
                _json1[k] = v
            elif type(v) == dict:
                _json1[k] = self.merge_dict(_json1[k], _json2[k])
        return _json1

    def check_and_create_by_dir(self, dir):
        if not os.path.exists(dir):
            logger.debug('创建目录:{}'.format(dir))
            os.makedirs(dir)

    def init_dir(self):
        if os.path.exists(constant.RUN_TMP_DIR):
            shutil.rmtree(constant.RUN_TMP_DIR)
        self.check_and_create_by_dir(constant.REPORT_DIR)
        self.check_and_create_by_dir(constant.RUN_TMP_DIR)
        if os.path.exists(constant.TMP_REPORT_DIR):
            shutil.rmtree(constant.TMP_REPORT_DIR)
        self.check_and_create_by_dir(constant.TMP_REPORT_DIR)
        self.check_and_create_by_dir(constant.TMP_REPORTS_DIR)

    def update_environment(self, result_tmp):
        environment = conf.settings.report_env
        xml = config.AllureXml()
        xml.append_child(environment)
        xml.save_path(os.path.join(result_tmp, 'environment.xml'))

    def update_parameters(self, behaviors):
        if type(behaviors) == dict:
            if 'parameters' in list(behaviors.keys()):
                behaviors['parameters'] = []
            elif 'children' in list(behaviors.keys()):
                for i in range(len(behaviors['children'])):
                    self.update_parameters(behaviors['children'][i])

    def update_behaviors(self, report_tmp):
        try:
            behaviors_path = os.path.join(report_tmp, 'data', 'behaviors.json')
            behaviors_obj = config.Json(behaviors_path)
            self.update_parameters(behaviors_obj.get_object())
            behaviors_obj.save()
        except Exception as e:
            logger.error(traceback.format_exc())

    def get_result(self):
        if self.result_folder is None or os.path.exists(self.result_folder) is False:
            self.check_and_create_by_dir(constant.TMP_RESULT_FOLDER)
            command = '{} generate {} -c -o {}'.format(constant.ALLURE_PATH, constant.TMP_REPORT_DIR, constant.TMP_RESULT_FOLDER)
            os.system(command)
            result_path = os.path.join(constant.TMP_RESULT_FOLDER, 'widgets', 'summary.json')
        else:
            result_path = os.path.join(self.result_folder, 'widgets', 'summary.json')
        _config = config.Json(result_path)
        statistic = _config.get('statistic')
        _time = _config.get('time')
        return statistic, _time

    def get_report(self,*result_folder):
        """
        生成allure报告
        :param result_folder:报告地址文件名
        :return:
        """
        self.result_folder = os.path.join(constant.BASE_DIR, *result_folder)
        command = '{} generate {} -c -o {}'.format(constant.ALLURE_PATH, constant.TMP_REPORT_DIR, self.result_folder)
        logger.info(command)
        self.update_environment(constant.TMP_REPORT_DIR)
        logger.info(subprocess.call(command, shell=True))

    def send_report_by_dingtalk(self):
        ...

    def convert_time(self, _time):
        result = ""
        if _time < 1000:
            return " {}毫秒 ".format(_time)
        else:
            _time = int(_time/1000)
        if _time > 3600:
            result += "{} 小时 ".format(_time // 3600)
            _time = _time % 3600
        if _time > 60:
            result += "{} 分钟 ".format(_time // 60)
            _time = _time % 60
        if _time != 0:
            result += "{} 秒".format(_time)
        return result

    def send_report_by_feishu(self, link, title=None, is_at_all=False):
        statistic, _time = self.get_result()
        logger.error(statistic)
        logger.error(_time)
        if len(_time) == 0:
            logger.error("请执行用例之后再进行报告发送")
            return
        created_at = self.unix_by_datetime(int(_time.get('start')/1000))
        duration = self.convert_time(_time.get('duration'))
        if title is None:
            title = '接口测试报告'
        if conf.settings.exists('robot'):
            robot = conf.settings.robot
            robot_type = robot.get('type', 'feishu')
            robot_func = robot.get('func', 'send_lark_report')
            if robot_type == "feishu":
                client = conf.settings.feishu
            if robot_func == 'send_lark_report':
                response = client.send_lark_report(title, created_at, duration, conf.settings.report_env, statistic,
                                                   link, is_at_all, other_elements=conf.settings.run.other_elements)
                logger.info(response.text)
            elif robot_func == 'send_lark_report_by_enterprise':
                response = client.send_lark_report_by_enterprise(title, created_at, duration, conf.settings.report_env,
                                                                 statistic,
                                                                 link, is_at_all,
                                                                 other_elements=conf.settings.run.other_elements)
                logger.info(response.text)
        else:
            logger.error('请配置robot参数')

    def send_feishu(self, access_token, secret, title, text):
        response = robot.Robot().send_feishu(access_token, secret, title, text)
        logger.info(response)

    def unix_by_datetime(self, unix_time):
        return datetime.datetime.fromtimestamp(int(float(str(unix_time)[:10]))).strftime("%Y-%m-%d %H:%M:%S")

    def load_yapi(self, name):
        yapi.Yapi(name).load_yapi()


if __name__ == "__main__":
    run = Run()
    run.run()
    run.get_report('allure')



from ..data import tmp
import re


class NewDict(dict):

    def __init__(self, *args, **kwargs):
        super(NewDict, self).__init__(*args, **kwargs)

    def get(self, *args):
        if len(args) <= 1:
            return super().get(args[0])
        else:
            result = []
            for i in args:
                result.append(super().get(i))
            return tuple(result)

    def merge(self, _dict: dict):
        self.replace_dict(self, _dict)

    def append_tmp(self, dpTmp):
        _id = self.get('id')
        tmp.tmp.append(_id, dpTmp)

    def replace_dict(self, _json1: dict, _json2: dict):
        for k in list(_json2.keys()):
            v = _json2[k]
            if k not in list(_json1.keys()) or type(v) != dict:
                _json1[k] = v
            elif type(v) == dict:
                _json1[k] = self.replace_dict(_json1[k], _json2[k])
        return _json1


def singleton(cls):
    """
    单例模式，使用方式： 在类上添加装饰器 @singleton
    @param cls:
    @return:
    """
    _instance = {}

    def _singleton(*args, **kwargs):
        # 先判断这个类有没有对象
        if cls not in _instance:
            _instance[cls] = cls(*args, **kwargs)  # 创建一个对象,并保存到字典当中
        # 将实例对象返回
        return _instance[cls]

    return _singleton


def hump2underline(hump_str):
    """
    驼峰形式字符串转成下划线形式
    :param hump_str: 驼峰形式字符串
    :return: 字母全小写的下划线形式字符串
    """
    # 匹配正则，匹配小写字母和大写字母的分界位置
    p = re.compile(r'([a-z]|\d)([A-Z])')
    # 这里第二个参数使用了正则分组的后向引用
    sub = re.sub(p, r'\1_\2', hump_str).lower()
    return sub


def underline2hump(underline_str):
    """
    下划线形式字符串转成驼峰形式
    :param underline_str: 下划线形式字符串
    :return: 驼峰形式字符串
    """
    # 这里re.sub()函数第二个替换参数用到了一个匿名回调函数，回调函数的参数x为一个匹配对象，返回值为一个处理后的字符串
    sub = re.sub(r'(_\w)', lambda x: x.group(1)[1].upper(), underline_str)
    return sub


def json_hump2underline(hump_json_str):
    """
    把一个json字符串中的所有字段名都从驼峰形式替换成下划线形式。
    注意点：因为考虑到json可能具有多层嵌套的复杂结构，所以这里直接采用正则文本替换的方式进行处理，而不是采用把json转成字典再进行处理的方式
    :param hump_json_str: 字段名为驼峰形式的json字符串
    :return: 字段名为下划线形式的json字符串
    """
    # 从json字符串中匹配字段名的正则
    # 注：这里的字段名只考虑由英文字母、数字、下划线组成
    attr_ptn = re.compile(r'"\s*(\w+)\s*"\s*:')
    # 使用hump2underline函数作为re.sub函数第二个参数的回调函数
    sub = re.sub(attr_ptn, lambda x: '"' + hump2underline(x.group(1)) + '" :', hump_json_str)
    return sub


def dict_hump2underline(hump_dict):
    """
    将dict的所有键内的驼峰转换为下划线形式字符串
    :param hump_dict: dict
    :return: 新的字典
    """
    new_dict = {}
    for key, value in hump_dict.items():
        new_dict[hump2underline(key)] = value
    return new_dict


def dict_value_hump2underline(hump_dict):
    """
    将dict的所有键内的驼峰转换为下划线形式字符串
    :param hump_dict: dict
    :return: 新的字典
    """
    if type(hump_dict) == str:
        return hump_dict
    elif type(hump_dict) == list:
        return [dict_value_hump2underline(i) for i in hump_dict]
    elif type(hump_dict) == dict:
        for i in list(hump_dict.keys()):
            new_key = hump2underline(i)
            hump_dict[new_key] = hump_dict[i]
            if new_key != i:
                del hump_dict[i]
        return hump_dict
    return hump_dict




import sys
from autoTestScheme import run, conf, logger


if __name__ == '__main__':
    argv = sys.argv
    if len(argv) == 4 and argv[1] in ['--feishu', 'feishu']:
        logger.info(conf.settings.feishu.send_message(argv[2], argv[3]).text)
    elif len(argv) == 2 and argv[1] in ['--help', '-h', 'help']:
        logger.info('--help/-h/help 查看命令集\n')
        logger.info('feishu/--feishu 发送飞书请求')
        logger.info('python -m autoTestScheme feishu 飞书标题  飞书内容')



'''
    用来转换特殊字符串
'''
import re
import copy
import json
import traceback
from autoTestScheme.common import logger


class Conversion(object):

    def __init__(self, this, json:dict):
        self.this = this
        self.json = json
        self.tmp_variable = {'false': False, 'True': True, 'true': True, 'False': False, 'null': None}
        self.start = r'\$(\d+)'
        self.end = r'\$'
        self.variable_regex = r"([a-zA-z].*)"
        self.function_regex = r"\{(\w+)\((.*?)\)\}"
        self.json_regex = r"JSON((\[.*?\]){1,20})"

    def re_dict(self):
        re_order_list = self.get_re_dict(self.json)
        re_order_list.sort()
        re_order_list = [i[1] for i in re_order_list]
        func_com = re.compile(self.start + self.function_regex)
        var_com = re.compile(self.start + self.variable_regex)
        json_com = re.compile(self.start + self.json_regex)
        for i in re_order_list:
            key = self.json
            for j in i:
                key = key[j]
            if len(func_com.findall(key)) > 0:
                new_value = self.re_sub_func(self.start + self.function_regex, key, self.replace_function)
            elif len(json_com.findall(key)) > 0:
                new_value = self.re_sub_func(self.start + self.json_regex, key, self.replace_json)
            elif len(var_com.findall(key)) > 0:
                new_value = self.re_sub_func(self.start + self.variable_regex, key, self.replace_variable)
            _json = self.json
            for jj in i[:-1]:
                _json = _json[jj]
            _json[i[-1]] = new_value
        return self.json

    def re_sub_func(self, regex, value, func):
        func_com = re.compile(regex)
        new_value = copy.deepcopy(value)

        _list = []
        _find_list = [k for k in func_com.finditer(value)]
        for k in _find_list[::-1]:
            pos = k.span()
            try:
                if (k.start() == 0 and len(value) == k.end()):
                    new_value = json.loads(func(k))
                else:
                    _str = list(new_value)
                    _str[pos[0]:pos[1]] = new_v = str(json.loads(func(k)))
                    new_value = ''.join(_str)
            except Exception as e:
                logger.error('{} 替换异常:{}'.format(value, traceback.format_exc()))
                raise e
        logger.debug('替换特殊字符串：{}，替换结果:{}'.format(value, new_value))
        return new_value

    def get_re_str(self, _str, loc):
        re_order_list = []
        re_func_result = re.compile(self.start + self.function_regex).findall(_str)
        re_val_result = re.compile(self.start + self.variable_regex).findall(_str)
        if len(re_func_result) > 0:
            pos = re_func_result[0][0]
            re_order_list.append([int(pos), loc])
        elif len(re_val_result) > 0:
            pos = re_val_result[0][0]
            re_order_list.append([int(pos), loc])
        return re_order_list

    def get_re_list(self, _list, loc):
        re_order_list = []
        for i in range(len(_list)):
            v = _list[i]
            _loc = loc + [i]
            if type(v) == dict:
                re_order_list += self.get_re_dict(v, _loc)
            if type(v) == list:
                re_order_list += self.get_re_list(v, _loc)
            elif type(v) == str:
                re_order_list += self.get_re_str(v, _loc)
        return re_order_list

    def get_re_dict(self, _dict, loc=[]):
        re_order_list = []
        for k, v in _dict.items():
            _loc = loc + [k]
            if type(v) == dict:
                re_order_list += self.get_re_dict(v, _loc)
            if type(v) == list:
                re_order_list += self.get_re_list(v, _loc)
            elif type(v) == str:
                re_order_list += self.get_re_str(v, _loc)
        return re_order_list

    def replace_function(self, value):
        if len(value.groups()) > 1:
            _find_list = value.groups()[1:]
        else:
            _find_list = value.groups()
        func = _find_list[0]
        _arg_list = []
        kwargs = {}
        if _find_list[1] == '':
            obj = self.this.get_func(func)
            if obj is not False:
                try:
                    from inspect import isgeneratorfunction
                    if isgeneratorfunction(obj) is False:
                        return json.dumps(obj())
                    else:
                        return json.dumps(next(obj()))
                except:
                    logger.error('replace value:{},error:{}'.format(value, traceback.format_exc()))
            return json.dumps(value.group())
        for arg in _find_list[1].split(','):
            if arg.find('=') != -1:
                _tmp = arg.split('=')
                if _tmp[1] in list(self.tmp_variable.keys()):
                    kwargs[_tmp[0]] = self.tmp_variable[_tmp[1]]
                else:
                    func_com = re.compile(self.end + self.function_regex)
                    var_com = re.compile(self.end + self.variable_regex)
                    _value = _tmp[1]
                    if len(func_com.findall(_tmp[1])) > 0:
                        _value = self.re_sub_func(self.end + self.function_regex, _value, self.replace_function)
                    elif len(var_com.findall(_tmp[1])) > 0:
                        _value = self.re_sub_func(self.end + self.variable_regex, _value, self.replace_variable)
                    kwargs[_tmp[0]] = _value
            else:
                if arg in list(self.tmp_variable.keys()):
                    _arg_list.append(self.tmp_variable[arg])
                else:
                    tmp = copy.deepcopy(arg)
                    func_com = re.compile(self.end + self.function_regex)
                    var_com = re.compile(self.end + self.variable_regex)
                    if len(func_com.findall(arg)) > 0:
                        tmp = self.re_sub_func(self.end + self.function_regex, tmp, self.replace_function)
                    elif len(var_com.findall(arg)) > 0:
                        tmp = self.re_sub_func(self.end + self.variable_regex, tmp, self.replace_variable)
                    try:
                        _arg_list.append(tmp)
                    except Exception:
                        _arg_list.append(tmp)
        arg = tuple(_arg_list)

        obj = self.this.get_func(func)
        if obj is not False:
            result = obj(*arg, **kwargs)
            result = json.dumps(result)
        else:
            result = json.dumps(value.group(0))
        return result

    def replace_json(self, value):
        if len(value.groups()) > 1:
            _find_list = value.groups()[1:]
        else:
            _find_list = value.groups()
        _find_list = _find_list[0]
        tmp = copy.deepcopy(self.json)
        for i in re.compile(r'\[(.*?)\]').findall(_find_list):
            if type(tmp) == list:
                tmp = tmp[int(i)]
            elif type(tmp) == str:
                if i.find(':') != -1:
                    tmp = tmp[int(i.split(':')[0]):int(i.split(':')[1])]
                else:
                    tmp = tmp[int(i)]
            else:
                tmp = tmp[i]
        return json.dumps(tmp)

    def replace_variable(self, value):
        if len(value.groups()) > 1:
            _find_list = value.groups()[1:]
        else:
            _find_list = value.groups()
        variable = _find_list[0]
        if variable.startswith('JSON') is True:
            tmp = copy.deepcopy(self.json)
            for i in re.compile(r'\[(.*?)\]').findall(variable):
                if type(tmp) == list:
                    tmp = tmp[int(i)]
                elif type(tmp) == str:
                    if i.find(':') != -1:
                        tmp = tmp[int(i.split(':')[0]):int(i.split(':')[1])]
                    else:
                        tmp = tmp[int(i)]
                else:
                    tmp = tmp[i]
            return json.dumps(tmp)
        obj = self.this.get_func(variable)
        if obj is not False:
            return json.dumps(obj)
        return json.dumps(variable)


#
ins = {
		"title":"网商-企业-已开户在线账户-修改手机号",
		"ins":{
            "customerAccountNo": "$1{get_customer_account(2,3,antb,1)}",
            "userId": "$2{get_user_id_by_customer_account_no($JSON[ins][customerAccountNo])}",
            "institutionCode": "ANTB",
            "name": "熊润",
            "nowMobile": "$2{get_update_moblie_by_customer_account_no($JSON[ins][customerAccountNo])}",
            "nowCode": "888888",
            "nowAuthNo": "$2{get_auth_no_by_update($JSON[ins][customerAccountNo])}",
            "originalCode": "888888",
            "a":{
                "a_son1": "$2{get_moblie_by_customer_account_no($JSON[ins][customerAccountNo])}",
                "a_son2": [
                    {
                        "a_son2_son": "$2JSON[ins][name]"
                    },
                    "$2JSON[ins][name]"
                ]
            },
            "originalMobile": "$2{get_moblie_by_customer_account_no($JSON[ins][customerAccountNo])}",
            "originalAuthNo": "$2{get_auth_no_by_original($JSON[ins][customerAccountNo])}",
            "tradeNo": "test_$2{current_subtle_unix()}"
        },
		"outs":{
            "status": 200,
            "errorCode": None,
            "message": None,
            "data": {
                "failReason": None,
                "result": None
            },
            "success": True
        }
	}


class Test(object):
    def __init__(self):
        self.current_subtle_unix_str = 123
        con = Conversion(self, ins)
        con.re_dict()
        print(con.json)

    def get_cash_sweep(self, v, k):
        return 123

    def get_user_id_by_customer_account_no(self, _str):
        return '---get_user_id_by_customer_account_no--{}'.format(_str)

    def get_customer_account(self, *args):
        return '560286448494204'

    def get_update_moblie_by_customer_account_no(self, _str):
        return '---get_update_moblie_by_customer_account_no--{}'.format(_str)

    def get_auth_no_by_update(self, _str):
        return '---get_auth_no_by_update--{}'.format(_str)

    def get_moblie_by_customer_account_no(self, _str):
        return '---get_moblie_by_customer_account_no--{}'.format(_str)

    def get_auth_no_by_original(self, _str):
        return '---get_auth_no_by_original--{}'.format(_str)

    def get_account_no_by_origin_cash_sweep_no(self,k):
        return 'ddddddddddddddddddddddd'

if __name__ == '__main__':
    Test()





# -*- coding: utf-8 -*-
from . import conf
from ._version import __version__
import os
import pytest
from _pytest.compat import NotSetType
from .data import tmp
from .common import constant, config as file, logger, common
from .run import json_name
from .pressure import case, httpBase


def pytest_addoption(parser):
    group = parser.getgroup(json_name, "distributed and subprocess testing")
    group.addoption(
        "--{}".format(json_name),
        action="store",
        help=(
            "provide an identifier shared amongst all workers as the value of "
            "the 'testrun_uid' fixture,\n\n,"
            "if not provided, 'testrun_uid' is filled with a new unique string "
            "on every test run."
        ),
    )


@common.singleton
class A(object):
    run_data = {}


a = A()


def pytest_configure(config):
    uuid = config.option.autoTestScheme
    if uuid is not None:
        _path = os.path.join(constant.RUN_TMP_DIR, uuid, 'case.json')
        a.run_data = file.Json(_path).get_object()
        conf.settings.run.test_env = a.run_data.get('__env')


def pytest_generate_tests(metafunc):
    _file = os.path.basename(metafunc.module.__file__).replace('.py', '')
    _func = metafunc.function.__name__
    conf.settings.run.test_env = a.run_data.get('__env')
    if 'filename' in dir(metafunc.cls):
        _file = metafunc.cls.filename
    if 'data' in metafunc.fixturenames and _file in list(a.run_data.keys()) \
            and _func in list(a.run_data[_file].keys()):
        data = a.run_data[_file][_func]
        _data = []
        for i in range(len(data)):
            _data.append(pytest.param(data[i], marks=pytest.mark.run(order=data[i]['__order'])))
        metafunc.parametrize("data", _data)
    else:
        metafunc.parametrize("data", [])


def pytest_runtest_setup(item):
    param = item.callspec.params.get('data')
    if param != NotSetType.token:
        _id = param.get('id')
        last = param.get('dependent')
        if param.get('last_dependent') is not None:
            last = param.get('last')
        if last is not None:
            item.callspec.params['data']['lastTmp'] = tmp.tmp.get_lastTmp(last)
            item.callspec.params['data']['dpTmp'] = tmp.tmp.get(last)
        if _id is not None:
            tmp.tmp.append_lastTmp(param.get('id'), param)

        logger.info('开始运行用例:{}'.format(param.get('title')))
        logger.info(item.callspec.params.get('data'))
        conf.settings.run.current_env = conf.settings.run.tag_env_list[param['tag']]
        conf.settings.run.current_tag = param['tag']


def pytest_runtest_call(__multicall__):
    """
    打印异常日志到日志文件内
    :param item:
    :return:
    """
    try:
        __multicall__.execute()
    except:
        logger.exception("pytest_runtest_call caught exception:")
        raise




import random
import time
import os
import sys
from decimal import Decimal
import json_tools
import allure
import pytest
import time
import datetime
import random
from decimal import Decimal
import requests
from faker import Faker
from . import conf
from .common import config, constant, common
from .common import logger
from .common.common import NewDict
from .data.conversion import Conversion
from autoTestScheme.configuration.sql import MySql
from autoTestScheme.configuration.request import requestBase


class Base(object):

    logger = logger
    tag_func_list = {}
    settings: conf.BaseDynaconf = conf.settings

    @property
    def faker(self):
        return Faker(locale='zh_CN')

    def convert_uppercase(self, string):
        """
        将字符串转换成大写
        """
        return string.upper()

    @property
    def not_repeat_string(self):
        """
        获取不重复字符串
        @return:
        """
        return '{}_{}_{}'.format(''.join(random.sample('zyxwvutsrqponmlkjihgfedcba', 4)),
                                 ''.join(random.sample('zyxwvutsrqponmlkjihgfedcba', 4)),
                                 str(float(time.time())))

    @classmethod
    def generate_phone(self):
        prefix = [
            '130', '131', '132', '133', '134', '135', '136', '137', '138', '139',
            '145', '147', '149', '150', '151', '152', '153', '155', '156', '157',
            '158', '159', '165', '171', '172', '173', '174', '175', '176', '177',
            '178', '180', '181', '182', '183', '184', '185', '186', '187', '188',
            '189', '191'
        ]

        # 随机取一个手机号前缀
        pos = random.randint(0, len(prefix) - 1)
        # 随机生成后8位数字
        suffix = str(int(time.time() * 1000))[-8:]
        # 拼接返回11位手机号
        return prefix[pos] + suffix

    @classmethod
    def current_subtle_unix(self):
        # 当前微妙时间
        return int(time.time() * 1000)

    def generate_email(self, domain='163.com'):
        """
        获取一个随机邮箱号
        :param domain:域名，默认163邮箱
        :return:
        """
        random_str = ''.join(random.sample('zyxwvutsrqponmlkjihgfedcba0123456789', random.randint(6, 20)))
        return 'test_{}@{}'.format(random_str, domain)

    @property
    def current_unix(self):
        # 当前时间
        return int(time.time())

    @property
    def current_subtle_unix_str(self):
        # 当前微妙时间
        # 等待1微妙防止重复
        time.sleep(0.001)
        return str(int(float(time.time()) * 100000))

    @property
    def current_subtle_str_unix(self):
        # 当前微妙时间（字符串类型）
        # 等待1微妙防止重复
        time.sleep(0.001)
        return str(int(float(time.time()) * 1000))

    def get_age_unix(self, num, is_positive=False):
        '''
            获取一个年龄超过n岁的时间
            is_positive 为false，取当前时间往前n年的时间
            is_positive 为true，取当前时间往后n年的时间
        '''
        num = int(num)
        _format = "%Y-%m-%d"
        today = datetime.date.today().strftime(_format)
        current_date = today.split()[0]
        y = current_date.split('-')[0]
        if is_positive is False:
            age = '{}-{}'.format(str(int(y) - num), '-'.join(current_date.split('-')[1:]))
        else:
            age = '{}-{}'.format(str(int(y) + num), '-'.join(current_date.split('-')[1:]))
        self.logger.debug(age)
        return int(time.mktime(time.strptime(age, _format)))

    @property
    def yesterday_start_unix(self):
        # 昨天开始时间
        return self.get_start_unix(1)

    def get_start_unix(self, day):
        '''
        获取n天前的开始时间
        @param day: n，提前n天
        @return:unix时间戳
        '''
        return int(
            time.mktime(time.strptime(str(datetime.date.today() - datetime.timedelta(days=int(day))), '%Y-%m-%d')))

    def get_start_date(self, day, format='%Y-%m-%d %H:%M:%S'):
        '''
        获取n天前的开始时间
        @param day: n，提前n天
        @param format: 时间格式
        @return:字符串时间
        '''
        return self.strp_unix_by_date(self.get_start_unix(day), format)

    def get_end_unix(self, day):
        '''
        获取n天前的结束时间
        @param day: n，提前n天
        @return:unix时间戳
        '''
        return self.get_start_unix(int(day) - 1) - 1

    def get_end_date(self, day, format='%Y-%m-%d %H:%M:%S'):
        '''
        获取n天前的结束时间
        @param day: n，提前n天
        @param format: 时间格式
        @return:字符串时间
        '''
        return self.strp_unix_by_date(self.get_end_unix(day), format)

    @property
    def yesterday_end_unix(self):
        # 昨天结束时间
        return int(time.mktime(time.strptime(str(datetime.date.today()), '%Y-%m-%d'))) - 1

    @property
    def today_start_unix(self):
        # 今天开始时间
        return int(time.mktime(time.strptime(str(datetime.date.today()), '%Y-%m-%d')))

    @property
    def today_end_unix(self):
        # 今天结束时间
        return int(time.mktime(time.strptime(str(datetime.date.today() + datetime.timedelta(days=1)), '%Y-%m-%d'))) - 1

    @property
    def tomorrow_start_unix(self):
        # 明天开始时间戳
        return int(time.mktime(time.strptime(str(datetime.date.today() + datetime.timedelta(days=1)), '%Y-%m-%d')))

    @property
    def tomorrow_end_unix(self):
        # 明天结束时间
        return int(time.mktime(time.strptime(str(datetime.date.today() + datetime.timedelta(days=2)), '%Y-%m-%d'))) - 1

    def strp_date_by_unix(self, date, format='%Y-%m-%d'):
        '''
        将字符串时间或datetime时间转换为unix时间
        @param date:字符串时间
        @param format:字符串时间格式，默认%Y-%m-%d,其他参考值%Y-%m-%d %H:%M:%S.%f
        @return:
        '''
        if type(date) == str:
            date = datetime.datetime.strptime(date, format)
        if format == '%Y-%m-%d %H:%M:%S.%f':
            return time.mktime(date.timetuple()) * 1e3 + date.microsecond / 1e3
        return time.mktime(date.timetuple())

    def current_local_date_str(self, format='%Y-%m-%d %H:%M:%S'):
        '''
        获取当前时间
        @param format: 返回格式，默认%Y-%m-%d %H:%M:%S
        @return:
        '''
        return datetime.datetime.now().strftime(format)

    def strp_unix_by_date(self, date, format='%Y-%m-%d %H:%M:%S'):
        '''
        将unix时间转换为字符串时间
        @param date:unix时间
        @param format:字符串时间格式，默认%Y-%m-%d,其他参考值%Y-%m-%d %H:%M:%S
        @return:
        '''
        # value为传入的值为时间戳(整形)，如：1332888820
        value = time.localtime(date)
        return time.strftime(format, value)

    def _sum(self, *args):
        '''
        求和，此函数解决float相加或相减
        @param args:
        @return:
        '''
        total = 0
        for i in args:
            total = Decimal(str(float(total))) + Decimal(str(float(i)))
        return float(total)

    def get_unique_identification(self):
        """
        获取唯一标识，多线程可用，用于性能测试使用
        """
        return 'test{}_{}_{}'.format(''.join(random.sample('zyxwvutsrqponmlkjihgfedcba', 4)),
                                              ''.join(random.sample('zyxwvutsrqponmlkjihgfedcba', 4)),
                                              str(float(time.time())))

    @pytest.fixture()
    def data_conversion(self, request):
        '''
            数据转换使用,将数据进行特殊转换
        '''
        if conf.settings._first_register is False:
            self._first_register = True
            auto_setup_01 = getattr(self, 'auto_setup_01', None)
            if auto_setup_01 is not None:
                auto_setup_01()
            for name in conf.settings.__dir__():
                if name.startswith("request"):
                    client = getattr(conf.settings, name)
                    for j in client.kwargs.get('setup_hook', []):
                        client.register_setup_hook(getattr(self, j))
                    for j in client.kwargs.get('teardown_hook', []):
                        client.register_teardown_hook(getattr(self, j))
            auto_setup_02 = getattr(self, 'auto_setup_02', None)
            if auto_setup_02 is not None:
                auto_setup_02()
        if 'data' in list(request.node.funcargs.keys()):
            param = request.node.funcargs.get('data')
        else:
            param = request.getfixturevalue('data')
        param = self.conversion(param)
        if param != {}:
            if len(self.settings.run.test_tags) > 1:
                param['title'] = '{}_{}'.format(conf.settings.run.tag_name_list[param['tag']], param['title'])
            allure.dynamic.title(param.get('title'))
            if param.get('story') is not None:
                allure.dynamic.story(param.get('story'))
            request.node.name = param.get('title')
        return param

    def teardown_class(self):
        ...

    def get_func(self, func):
        if func in dir(self):
            return eval('self.{}'.format(func))
        return False

    def conversion(self, param, title=''):
        con = Conversion(self, param)
        con.re_dict()
        param = con.json
        self.excute_dynamic(param.get('issue'), 'issue')
        self.excute_dynamic(param.get('link'), 'link')
        self.excute_dynamic(param.get('testcase'), 'testcase')
        return self.get_my_dict(param)

    def get_my_dict(self, param):
        '''
        将字典类添加一个新的获取方法(gets)，改方法可以一次性获取多个值
        @param param: 字典类
        @return: MyDict类
        '''
        return NewDict(param)

    def check_response(self, response, outs):
        '''
            数据对比，常用于在判断请求结果与预期的校验
        '''
        compare = json_tools.diff(outs, response)
        result = {'remove':[], 'add':[], 'replace':[]}
        for i in compare:
            _type = ''
            if 'remove' in list(i.keys()):
                _type = 'remove'
                i = i['remove']
            elif 'add' in list(i.keys()):
                _type = 'add'
                i = i['add']
            elif 'replace' in list(i.keys()):
                _type = 'replace'
                i['实际'] = i['value']
                i['预期'] = i['prev']
                i['路径'] = i['replace']
                if 'details' in i:
                    if i['details'] == 'type':
                        i['说明'] = '类型错误'
                    del i['details']
                del i['value']
                del i['prev']
                del i['replace']
            result[_type].append(i)
        if len(result['add']) > 0:
            self.logger.warning('校验的内容中增加的字段:{}'.format(' ; '.join(result['add'])))
        msg = []
        if len(result['remove']) > 0:
            msg.append("校验的内容中被删除的字段:{}".format(' ; '.join(result['remove'])))
        if len(result['replace']) > 0:
            msg.append("校验的内容中被修改的字段及内容:{}".format(result['replace']))
        assert len(msg) == 0, '比较结果:{}'.format(';'.join([str(i) for i in msg]))

    def check_inclusion_relation(self, a, b):
        '''
            a,b都是dict类型
            判断b是否包含a内的所有元素
        '''
        for i in list(a.items()):
            if i not in list(b.items()):
                self.logger.error('不包含元素:{},数据:\na:{},\nb:{}'.format(i, a, b))
                return False
        return True

    def excute_dynamic(self, dynamic, name):
        dynamic_list = {}
        dynamic_list['issue'] = allure.dynamic.issue
        dynamic_list['link'] = allure.dynamic.link
        dynamic_list['testcase'] = allure.dynamic.testcase
        if dynamic is not None:
            if type(dynamic) == dict:
                for a, b in dynamic.items():
                    dynamic_list[name](b, a)
            elif type(dynamic)  == list:
                for i in dynamic:
                    if type(i) == list:
                        dynamic_list[name](*i)
                    dynamic_list[name](i)

    def check_response_by_sql(self, response, outs):
        '''
            比较返回与数据库，与check_response的区别在于会将response与outs的key转换为下划线形式字符串
        '''
        response = common.dict_value_hump2underline(response)
        outs = common.dict_value_hump2underline(outs)
        return self.check_response(response, outs)

    def conver_decimal(self, data:dict) -> dict:
        """
        将字典内的Decimal类型字段转换未float
        @param data:
        @return:
        """
        for i in list(data.keys()):
            if type(data[i]) == Decimal:
                data[i] = float(data[i])
        return data



import requests
import os
from autoTestScheme import conf, logger
from autoTestScheme.common import constant, config
from autoTestScheme.common.common import hump2underline


class Yapi(object):
    """
    yapi
    """

    def __init__(self, name):
        if conf.settings.exists(name):
            self.yapi_config = conf.settings.get(name)
            assert self.yapi_config.get('case_folder') is not None, "case_folder 配置不存在"
            assert self.yapi_config.get('api_folder') is not None, "api_folder 配置不存在"
            assert self.yapi_config.get('data_folder') is not None, "data_folder 配置不存在"
        else:
            assert False, "不存在配置:{}".format(name)

    def send_yapi_request(self, url, data):
        """
        发送yapi请求
        :param url:
        :param data:
        :return:
        """
        if self.yapi_config.base_url.endswith("/") is False:
            self.yapi_config.base_url += "/"
        data['token'] = self.yapi_config.token
        url = self.yapi_config.base_url + url
        response = requests.get(url, data).json()
        logger.debug('url:{}, data:{}\n, response:{}'.format(url, data, response))
        assert response['errcode'] != 40011, 'token错误'
        return response['data']

    def load_yapi(self):
        """
        加载Yapi用例、api、代码
        :return:
        """
        case_id_list = self.yapi_config.get('case_id_list', [])
        if len(case_id_list) == 0:
            module_list = self.yapi_config.get('module_list')
            limit = self.yapi_config.get('limit', 10000)
            if module_list is None:
                data = {'project_id': self.yapi_config.project_id, 'limit': limit}
                case_id_list = [i['_id'] for i in self.send_yapi_request("api/interface/list", data)]
            else:
                for module in module_list:
                    data = {'catid': module, 'limit': limit}
                    case_id_list += [i['_id'] for i in self.send_yapi_request("api/interface/list_cat", data)['list']]
        self.load_yapi_by_id_list(case_id_list, conf.settings.run.test_tags)

    def load_yapi_by_id_list(self, id_list, tag=[]):
        """
        根据用例ID列表加载yapi用例
        :param id_list:
        :param tag:
        :return:
        """
        if len(id_list) == 0:
            logger.error("yapi加载错误，请确认项目ID、url是否配置正确")
            return
        module_info = {i['_id']: i['name'] for i in self.send_yapi_request("api/interface/getCatMenu", {'project_id': self.yapi_config.project_id})}
        project_name = self.send_yapi_request("api/project/get", {})['name']
        api_folder = os.path.join(constant.BASE_DIR, *self.yapi_config.api_folder)
        data_folder = os.path.join(constant.BASE_DIR, *self.yapi_config.data_folder)
        case_folder = os.path.join(constant.BASE_DIR, *self.yapi_config.case_folder)
        for _id in id_list:
            api_info = self.send_yapi_request("api/interface/get", {'id': _id})

            _path = os.path.split(api_info['path'])
            api_id = hump2underline(_path[-1])
            if len(_path) > 1 and _path[-2].startswith('v') is False:
                api_id += '_{}'.format(os.path.split(_path[-2])[-1])
            module_name = module_info[api_info['catid']]
            ins = self.get_yapi_ins(api_info)
            self.write(api_id, api_info['title'].strip(), api_info['method'], api_info['path'], project_name, module_name,
                       tag, api_folder, data_folder, case_folder, ins, base=self.yapi_config.base)

    def get_yapi_ins(self, api_info):
        """
        获取yapi的入参
        :param api_info:
        :return:
        """
        ins = {}
        for i in api_info['req_body_form']:
            example = i.get('example')
            if example is None:
                if i.get('type', 'text') == 'text':
                    example = str(3)
            ins[i['name']] = example
        return ins

    def write(self, api_id, title, method, path, project_name, module_name, tag, api_folder, data_folder, case_folder,
              ins={}, outs={}, base="from base.base import TestBase"):
        """
        写入
        :param api_id: api的ID号
        :param title: api标题
        :param method: 请求方式
        :param project_name: 项目名
        :param module_name: 模块名
        :param path: 请求路径
        :param tag: 当前环境
        :param api_folder: api的路径
        :param data_folder: data的路径
        :param case_folder: case的路径
        :param ins: 输入
        :param outs: 输出
        :return:
        """
        data = {"id": api_id, "title": title, "method": method, "params": {}, "data": {},
                "headers": {}, "path": path}
        # 写入API
        api_path = os.path.join(api_folder, api_id + ".json")
        case_name = 'test_{}'.format(api_id)
        if os.path.exists(api_path) is False:
            logger.info("开始写入api:路径({}), 标题({}), api_id({})".format(api_path, title, api_id))
            config.Json(api_path).set_object(data)
        # 写入case
        case_path = os.path.join(case_folder, case_name + '.py')
        if os.path.exists(case_path) is False:
            case_code = self.get_case_code(module_name, title, api_id, base)
            logger.info("开始写入case:路径({}), 标题({}), api_id({})".format(case_path, title, api_id))
            with open(case_path, 'w') as f:
                f.write(case_code)

        # 写入data数据
        data_path = os.path.join(data_folder, case_name + ".json")
        if os.path.exists(data_path) is False:
            logger.info("开始写入data:路径({}), 标题({}), api_id({})".format(data_path, title, api_id))
            data = []
            data.append({"__file": case_name, "__func": "test_01", "__dependent_class": None})
            data.append({"title": title, "tags": tag, "ins": ins, "outs": outs})
            config.Json(data_path).set_object(data)

    def get_case_code(self, epic, feature, api_id, base="from base.base import TestBase"):
        return """
import allure
{}


@allure.epic("{}")
@allure.feature("{}")
class TestCase(TestBase):

    def test_01(self, data_conversion, data):
        ins, outs = data_conversion.get('ins', 'outs')
        response = self.send_post_request("{}", ins)
        self.check_response(response, outs)\n""".format(base, epic, feature, api_id)



import re


def hump2underline(hump_str):
    """
    驼峰形式字符串转成下划线形式
    :param hump_str: 驼峰形式字符串
    :return: 字母全小写的下划线形式字符串
    """
    # 匹配正则，匹配小写字母和大写字母的分界位置
    p = re.compile(r'([a-z]|\d)([A-Z])')
    # 这里第二个参数使用了正则分组的后向引用
    sub = re.sub(p, r'\1_\2', hump_str).lower()
    return sub


def underline2hump(underline_str):
    """
    下划线形式字符串转成驼峰形式
    :param underline_str: 下划线形式字符串
    :return: 驼峰形式字符串
    """
    # 这里re.sub()函数第二个替换参数用到了一个匿名回调函数，回调函数的参数x为一个匹配对象，返回值为一个处理后的字符串
    sub = re.sub(r'(_\w)', lambda x: x.group(1)[1].upper(), underline_str)
    return sub


def json_hump2underline(hump_json_str):
    """
    把一个json字符串中的所有字段名都从驼峰形式替换成下划线形式。
    注意点：因为考虑到json可能具有多层嵌套的复杂结构，所以这里直接采用正则文本替换的方式进行处理，而不是采用把json转成字典再进行处理的方式
    :param hump_json_str: 字段名为驼峰形式的json字符串
    :return: 字段名为下划线形式的json字符串
    """
    # 从json字符串中匹配字段名的正则
    # 注：这里的字段名只考虑由英文字母、数字、下划线组成
    attr_ptn = re.compile(r'"\s*(\w+)\s*"\s*:')
    # 使用hump2underline函数作为re.sub函数第二个参数的回调函数
    sub = re.sub(attr_ptn, lambda x: '"' + hump2underline(x.group(1)) + '" :', hump_json_str)
    return sub

def dict_hump2underline(hump_dict):
    """
    将dict的所有键内的驼峰转换为下划线形式字符串
    :param hump_dict: dict
    :return: 新的字典
    """
    new_dict = {}
    for key, value in hump_dict.items():
        new_dict[hump2underline(key)] = value
    return new_dict

def dict_value_hump2underline(hump_dict):
    """
    将dict的所有键内的驼峰转换为下划线形式字符串
    :param hump_dict: dict
    :return: 新的字典
    """
    if type(hump_dict) == str:
        return hump_dict
    elif type(hump_dict) == list:
        return [dict_value_hump2underline(i) for i in hump_dict]
    elif type(hump_dict) == dict:
        for i in list(hump_dict.keys()):
            new_key = hump2underline(i)
            hump_dict[new_key] = hump_dict[i]
            if new_key != i:
                del hump_dict[i]
        return hump_dict
    return hump_dict



import gevent.monkey
gevent.monkey.patch_all()
from . import common
from .._version import __version__
from loguru import logger


debug = logger.debug
info = logger.info
error = logger.error
warning = logger.warning
exception = logger.exception


logger.info("autoScheme Version:{}".format(__version__))







import os
import time
import hmac
import json
import base64
from hashlib import sha256
import requests
from autoTestScheme.common import logger
from dingtalkchatbot.chatbot import DingtalkChatbot

class Feishu(object):


    def __init__(self, access_token, secret, app_id=None, app_secret=None, app_verification_token=None, chat_id=None,
                 url="https://open.feishu.cn"):
        """
        :param access_token: 机器人access_token
        :param secret: 机器人access_token
        :param app_id: 应用ID（企业机器人专用）
        :param app_secret: 应用secret（企业机器人专用）
        :param app_verification_token: 应用verification_token（企业机器人专用）
        :param chat_id: 企业机器人所在群ID
        :param url:
        """
        self.access_token = access_token
        self.secret = secret
        self.url = url
        self.APP_ID = app_id
        self.APP_SECRET = app_secret
        self.APP_VERIFICATION_TOKEN = app_verification_token
        self.chat_id = chat_id

    def get_url(self, path):
        if path.endswith("/") is False:
            path += "/"
        return self.url + path

    def send(self, data):
        headers = {'Content-Type': 'application/json'}
        url = self.get_url("/open-apis/bot/v2/hook/" + self.access_token)
        logger.info(data)
        return requests.request("POST", url, headers=headers, data=json.dumps(data), verify=False)

    def send_message(self, title, text):
        data = {
            "msg_type": "post",
            "content": {
                "post": {
                    "zh_cn": {
                        "title": title,
                        "content": [[{"tag": "text", "text": text}]
                        ]
                    }
                }
            }
        }
        if self.chat_id is not None:
            return self.send_message_by_enterprise(self.get_tenant_access_token(), self.chat_id, data)
        return self.send(data)

    def get_fileds(self, _list):
        fields = []
        i = 0
        for value in _list:
            if i % 2 == 0:
                fields.append({"is_short": False, "text": {"tag": "lark_md", "content": ""}})
            fields.append(
                {"is_short": True, "text": {"tag": "lark_md", "content": "**{}**\n{}".format(value[0], value[1])}})
            i += 1
        return fields

    def send_lark_report(self, *args, **kwargs):
        """
        发送lark报告
        :param title: 标题
        :param created_at: 创建时间
        :param duration: 总用时
        :param env: 环境
        :param summary: 用例执行情况
        :param link: 报告地址
        :param is_at_all: 是否@所有人
        :param at_list: @列表
        :return:
        """
        data = self.get_feishu_report_data(*args, **kwargs)
        if self.secret != '' and self.secret is not None:
            timestamp = str(round(time.time()))
            key = '{}\n{}'.format(timestamp, self.secret)
            key_enc = key.encode('utf-8')
            hmac_code = hmac.new(key_enc, digestmod=sha256).digest()
            sign = base64.b64encode(hmac_code).decode('utf-8')
            data["timestamp"] = timestamp
            data["sign"] = sign
        return self.send(data)

    def send_lark_report_by_enterprise(self, *args, **kwargs):
        data = self.get_feishu_report_data(*args, **kwargs)
        return self.send_message_by_enterprise(self.get_tenant_access_token(), self.chat_id, data)

    def get_feishu_report_data(self, title, created_at, duration, env, summary, link, is_at_all=False, at_list=[],
                         other_elements=[]):
        logger.error(other_elements)
        header = {"title": {"tag": "plain_text", "content": title}, "template": "orange"}
        fields1 = self.get_fileds(env)
        fields2 = self.get_fileds([['创建时间', created_at], ['执行总时长', duration]])
        fields3_data = []
        for i in [('总用例数', 'total'), ('成功', 'passed'), ('失败', 'failed'), ('代码异常', 'broken'), ('跳过', 'skipped')
            , ('未知', 'unknown')]:
            if summary.get(i[1]) is not None and summary.get(i[1]) > 0:
                fields3_data.append([i[0], summary.get(i[1])])
        fields3 = self.get_fileds(fields3_data)
        elements = []
        elements.append({"tag": "div", "fields": fields1, "text": {"tag": "lark_md", "content": "**环境详情**"}})
        elements.append({'tag': 'hr'})
        elements.append({"tag": "div", "fields": fields2, "text": {"tag": "lark_md", "content": "**用时详情**"}})
        elements.append({'tag': 'hr'})
        elements.append({"tag": "div", "fields": fields3, "text": {"tag": "lark_md", "content": "**执行详情**"}})
        for element in other_elements:
            elements.append({'tag': 'hr'})
            fields_tmp = self.get_fileds(element['fields'])
            elements.append({"tag": "div", "fields": fields_tmp,
                             "text": {"tag": "lark_md", "content": "**{}**".format(element['title'])}})
        elements.append({"actions": [{"tag": "button", "text": {"content": "查看详情", "tag": "lark_md"},
                                      "url": link, "type": "default", "value": {}}], "tag": "action"})
        if is_at_all is True:
            elements.append({"tag": "markdown", "content": "<at id=all></at>"})
        elif len(at_list) > 0:
            content = ""
            for user in at_list:
                content += "<at "
                if user.get('user_id') is not None:
                    content += "id='{}'".format(user.get('user_id'))
                content += " >"
                if user.get('user_name') is not None:
                    content += " {} ".format(user.get('user_name'))
                content += "</at>"
            elements.append({"tag": "markdown", "content": content})
        data = {}
        data["msg_type"] = "interactive"
        config = {"wide_screen_mode": True, "enable_forward": True}
        data['card'] = {'config': config, 'elements': elements, 'header': header}
        return data

    def send_message_by_enterprise(self, token, chat_id, req_body):
        logger.error(req_body)
        params = {'receive_id_type': 'chat_id'}
        req_body['receive_id'] = chat_id
        if req_body['msg_type'] == 'interactive':
            req_body['content'] = json.dumps(req_body['card'])
        return self.send_post_by_enterprise("/open-apis/im/v1/messages", data=req_body, token=token, params=params)

    def get_tenant_access_token(self):
        data = {"app_id": self.APP_ID, "app_secret": self.APP_SECRET}
        return self.send_post_by_enterprise("/open-apis/auth/v3/tenant_access_token/internal", data).json().get("tenant_access_token", "")

    def send_post_by_enterprise(self, path, data, token=None, params=None):
        headers = {'Content-Type': 'application/json; charset=utf-8'}
        if token is not None:
            headers['Authorization'] = "Bearer " + token
        logger.error("path:{}, \ndata:{}, \nheaders:{}, params:{}".format(path, data, headers, params))
        url = self.get_url(path)
        return requests.request("POST", url, params=params, headers=headers, data=json.dumps(data), verify=False)


class Robot(object):

    def __init__(self):
        pass

    def get_markdown(self, created_at, env, environment, summary, title=None, link=None, is_feishu=False):
        msg = ''
        
        if title is not None:
            msg += '## {}\n'.format(title)
            msg += '___\n'
        for i in env:
            msg += '#### <font color="#000000">{}</font><br/>{}\n'.format(i[0], i[1])
        msg += '#### <font color="#000000">创建时间:</font><br/>{}\n'.format(created_at)
        msg += '#### <font color="#000000">语言环境:</font><br/>\n'
        son1 = '' if is_feishu is True else '+ '
        for key, value in environment.items():
            msg += '{}<font color="#00FF7F">{}:</font>{}\n'.format(son1, key, value)
        msg += '#### <font color="#000000">测试结果:</font><br/>\n'
        language = {}
        language['num_tests'] = '<font color="#00FF7F">总条数:</font><br/>'
        language['passed'] = '<font color="#00FF7F">成功:</font><br/>'
        language['duration'] = '<font color="#00FF7F">执行时长(秒):</font><br/>'
        language['failed'] = '<font color="#FF0000">执行失败:</font><br/>'
        language['skipped'] = '<font color="#FFE4C4">跳过:</font><br/>'
        language['total'] = '<font color="#FFE4C4">总条数:</font><br/>'
        language['broken'] = '<font color="#FFE4C4">代码异常:</font><br/>'
        language['unknown'] = '<font color="#FFE4C4">未知:</font><br/>'
        son2 = '' if is_feishu is True else '- '
        for key, value in summary.items():
            if key in list(language.keys()):
                key = language[key]
            msg += '{}{}{}\n'.format(son2, key, value)
        if link is not None:
            msg += '#### <font color="#000000">详情</font><br/>:[点此跳转]({})'.format(link)
        return msg

    def execute_robot(self, robot_type, access_token, secret, title, created_at, env, environment, summary, link, is_at_all=False, at_list=[]):
        self.robot_type = robot_type
        self.access_token = access_token
        self.secret = secret
        self.title = title
        self.created_at = created_at
        self.env = env = env
        self.environment = environment
        self.summary = summary
        self.link = link
        self.is_at_all = is_at_all
        self.at_list = at_list
        if robot_type == 'dingtalk':
            return self.send_dingtalk(access_token, secret, title, created_at, env, environment, summary, link, is_at_all, at_list)
        elif robot_type == 'feishu_report':
            return self.send_feishu_report(access_token, secret, title, created_at, env, environment, summary, link, is_at_all, at_list)

    def send_dingtalk(self, access_token, secret, title, created_at, env, environment, summary, link, is_at_all=False, at_list=[]):
        webhook = 'https://oapi.dingtalk.com/robot/send?access_token={}'.format(access_token) 
        xiaoding = DingtalkChatbot(webhook, secret=secret) # 方式二：勾选“加签”选项时使用（v1.5以上新功能） 
        msg = self.get_markdown(created_at, env, environment, summary, title, link)
        return xiaoding.send_markdown(title, msg, is_at_all=is_at_all, at_mobiles=at_list)

    def send_feishu_report(self, access_token, secret, title, created_at, env, environment, summary, link, is_at_all=False, at_list=[]):
        if 'http' in access_token:
            url = access_token
        else:
            url = 'https://open.feishu.cn/open-apis/bot/v2/hook/{}'.format(access_token)
        # data = {}
        header = {"title": {"tag": "plain_text", "content": title}, "template": "blue"}
        print(env)
        fields1 = []
        i = 0
        for value in env:
            if i % 2 == 0:
                fields1.append({"is_short": False, "text": {"tag": "lark_md", "content": ""}})
            fields1.append({"is_short": True, "text": {"tag": "lark_md", "content": "**{}**\n{}".format(value[0], value[1])}})
            i += 1
        fields2 = []
        fields2.append({"is_short": True, "text": {"tag": "lark_md", "content": "**创建时间:**\n{}".format(created_at)}})
        fields2.append({"is_short": True, "text": {"tag": "lark_md", "content": "**执行总时长:**\n{}".format(created_at)}})
        fields3 = []
        fields3.append({"is_short": False, "text": {"tag": "lark_md", "content": ""}})
        fields3.append({"is_short": True, "text": {"tag": "lark_md", "content": "**总用例数:**\n{}".format(summary.get('total'))}})
        fields3.append({"is_short": True, "text": {"tag": "lark_md", "content": "**成功:**\n{}".format(summary.get('passed'))}})
        fields3.append({"is_short": False, "text": {"tag": "lark_md", "content": ""}})
        fields3.append({"is_short": True, "text": {"tag": "lark_md", "content": "**失败:**\n{}".format(summary.get('failed'))}})
        fields3.append({"is_short": True, "text": {"tag": "lark_md", "content": "**代码异常:**\n{}".format(summary.get('broken'))}})
        fields3.append({"is_short": False, "text": {"tag": "lark_md", "content": ""}})
        fields3.append({"is_short": True, "text": {"tag": "lark_md", "content": "**跳过:**\n{}".format(summary.get('skipped'))}})
        fields3.append({"is_short": True, "text": {"tag": "lark_md", "content": "**未知:**\n{}".format(summary.get('unknown'))}})
        elements = []
        elements.append({"tag": "div", "fields": fields1, "text": {"tag": "lark_md", "content": "**环境详情**"}})
        elements.append({'tag': 'hr'})
        elements.append({"tag": "div", "fields": fields2, "text": {"tag": "lark_md", "content": "**执行时间**"}})
        elements.append({'tag': 'hr'})
        elements.append({"tag": "div", "fields": fields3, "text": {"tag": "lark_md", "content": "**执行详情**"}})
        elements.append({"actions": [{"tag": "button", "text": {"content": "查看详情", "tag": "lark_md"},
                                             "url": link, "type": "default", "value": {}}], "tag": "action"})


        data = {}
        data["msg_type"] = "interactive"
        config = {"wide_screen_mode": True, "enable_forward": True}
        data['card'] = {'config': config, 'elements': elements, 'header': header}
        headers = {'Content-Type': 'application/json'}
        if secret != '' and secret is not None:
            timestamp = str(round(time.time()))
            key = '{}\n{}'.format(timestamp, secret)
            print(key)
            key_enc = key.encode('utf-8')
            hmac_code = hmac.new(key_enc, digestmod=sha256).digest()
            sign = base64.b64encode(hmac_code).decode('utf-8')
            data["timestamp"] = timestamp
            data["sign"] = sign
        data['at'] = {}
        data['at']["isAtAll"] = is_at_all
        print(url, data)
        response = requests.request("POST", url, headers=headers, data=json.dumps(data), verify=False)
        return response.text
    
    def send_feishu(self,  access_token, secret, title, text):
        headers = {'Content-Type': 'application/json'}
        if 'http' in access_token:
            url = access_token
        else:
            url = 'https://open.feishu.cn/open-apis/bot/v2/hook/{}'.format(access_token)
        data = {
            "msg_type": "post",
            "content": {
                "post": {
                    "zh_cn": {
                        "title": title,
                        "content": [[{"tag": "text", "text": text}]
                        ]
                    }
                }
            }
        }
        response = requests.request("POST", url, headers=headers, data=json.dumps(data), verify=False)
        return response.text
    

robot = Robot()



import os
from ..common import config, constant, logger


class Api(object):

    def __init__(self):
        self.api_data = {}

    def read_api_folder(self, *folder_name):
        if isinstance(folder_name, tuple) is False:
            folder_name = (folder_name,)
        folder = os.path.join(constant.BASE_DIR, *folder_name)
        for root, dirs, files in os.walk(folder):
            for name in files:
                _path = os.path.join(root, name)
                if name.endswith('.json') is True:
                    _data = config.Json(_path).get_object()
                    if _data.get('id') is not None:
                        self.register_api(_data)
                elif name.endswith('.proto') is True:
                    self.register_api(name[:-6], _path)

    def register_api(self, _data):
        _id = _data.get('id')
        if _id in list(self.api_data.keys()):
            logger.error("api({})重复注册".format(_id))
        self.api_data[_id] = _data

    def register_proto(self, _id, _path):
        if _id in list(self.api_data.keys()):
            logger.error("api({})重复注册".format(_id))
        self.api_data[_id] = _path

    def get_api(self, api_id):
        """
        返回api数据
        :param api_id:
        :return:
        """
        if api_id in list(self.api_data.keys()):
            return self.api_data[api_id]
        assert False, "api({})未注册".format(api_id)





from locust import HttpUser
from .case import Case
from base.base import TestBase


class HttpCase(HttpUser, Case, TestBase):

    def __init__(self, *args, **kwargs):
        super(HttpCase, self).__init__(*args, **kwargs)
        Case.__init__(self)

    def start(self, *args, **kwargs):
        self.singleton.run()
        super().start(*args, **kwargs)

    def on_locust_setup(self):
        super().on_locust_setup()
        self.session.catch_response = True
        self.session.session = self.client





class Tmp(object):
    __species = None

    def __new__(cls, *args, **kwargs):
        if not cls.__species:
            cls.__species = object.__new__(cls)
        return cls.__species

    def __init__(self):
        self.tmp = {}
        self.lastTmp = {}

    def append(self, key, value):
        self.tmp[key] = value

    def append_lastTmp(self, key, value):
        self.lastTmp[key] = value

    def get_lastTmp(self, key):
        if key not in list(self.lastTmp.keys()):
            return
        return self.lastTmp[key]

    def get(self, key):
        if key not in list(self.tmp.keys()):
            return
        return self.tmp.get(key)

tmp = Tmp()



import os
import sys
import allure
import copy
import time
import inspect
import requests
import json
from allure_commons._allure import Attach
from allure import attachment_type
from urllib.parse import urlencode
from . import api
from ..common import config, constant, logger


class Attachs(Attach):

    def __init__(self, *args, **kwargs):
        super().__call__(*args, **kwargs)

    def __enter__(self):
        ...

    def __exit__(self, exc, value, tb):
        ...


class requestBase(api.Api):

    def __init__(self):
        super().__init__()
        self.setup_hook_list = []
        self.teardown_hook_list = []
        self.logger = logger
        self.hook_switch = {}
        self.session = requests.Session()
        self.base_url = None # 域名
        self.catch_response = False # locust的时候使用
        self.kwargs = {} # request配置文件内的参数

    def set_session(self, client):
        self.session = client

    def register_setup_hook(self, hook):
        """
        注册前置钩子
        :param hook:
        :return:
        """
        self.hook_switch[hook.__name__] = True
        self.setup_hook_list.append(hook)

    def register_teardown_hook(self, hook):
        """
        注册后置钩子
        :param hook:
        :return:
        """
        self.hook_switch[hook.__name__] = True
        self.teardown_hook_list.append(hook)

    def get_url(self, path):
        '''
            获取网址
        '''
        assert self.base_url is not None, '未定义base_url'

        if self.base_url.endswith('/'):
            self.base_url = self.base_url[:-1]
        url = self.base_url + path
        return url

    def send(self, api_id, is_json=True, title=None, **kwargs):
        request = self.get_api(api_id)
        request['is_json'] = is_json
        if title is not None:
            request['title'] = title
        request.update(kwargs)
        response = self._excute(request)
        if is_json is True:
            return response.json()
        return response

    def close_hook(self, key=None):
        if key is None:
            for k in self.hook_switch:
                self.hook_switch[key] = False
        else:
            self.hook_switch[key] = False

    def open_hook(self, key=None):
        if key is None:
            for k in self.hook_switch:
                self.hook_switch[key] = True
        else:
            self.hook_switch[key] = True

    def execute(self, request):
        """
            发送请求
        """
        for hook in self.setup_hook_list:
            if self.hook_switch[hook.__name__] is True:
                self.excute_kwargs(hook, request)
        request['title'] = request.get('title') if request.get('title') is not None else request.get('path')

        _name = request['title'].format(**request)
        with allure.step(_name):
            allure.attach(json.dumps(request, sort_keys=True, ensure_ascii=False, indent=4, separators=(',', ':')),
                     name="请求体", attachment_type=attachment_type.JSON)
            self.logger.debug('发送请求：{}'.format(request['title']))
            response = self._excute(request)
            for hook in self.teardown_hook_list:
                if self.hook_switch[hook.__name__] is True:
                    self.excute_kwargs(hook, request)
            return response

    def excute_kwargs(self, hook, request):
        kwarg = {}
        kwargs = {'request': request, 'session': self.session, 'kwargs': self.kwargs}
        for i in inspect.getfullargspec(hook).args:
            if i in ['self', 'cls']:
                continue
            kwarg[i] = kwargs[i]
        hook(**kwarg)
    
    def copy_request(self, request):
        new_request = {}
        if 'files' not in request:
            new_request = copy.deepcopy(request)
        else:
            for key in request:
                if key == 'files':
                    new_request['files'] = request['files']
                else:
                    new_request[key] = copy.deepcopy(request[key])
        return new_request

    def _excute(self, request):
        current_time = time.time()
        new_request = self.copy_request(request)
        new_request['url'] = self.get_url(new_request.get('path'))
        if 'path' in new_request:
            del new_request['path']
        new_request['verify'] = False
        if self.catch_response is True:
            new_request['url'] = new_request['url'] + '?' + urlencode(new_request['params'])
            new_request['name'] = new_request['title']
            del new_request['params']
            new_request['catch_response'] = True
        for i in ['id', 'title']:
            if i in new_request:
                del new_request[i]
        result = self.session.request(**new_request)
        response = result.text.replace('\n', '').replace('\r', '').replace('  ', '')
        self.logger.debug('请求参数：method：{}，url：{}, \nparams:{}, headers:{}, \ndata:{}, json:{}, \n返回：{},时间:{}, '
                          '状态码:{}'.format(new_request.get('method'), new_request.get('url'), new_request.get('params'),
                                            new_request.get('headers'), new_request.get('data'), new_request.get('json'),
                                            response, time.time() - current_time, result.status_code))
        if self.catch_response is True:
            return result
        return result

    def qs_parse(self, data):
        for key in list(data.keys()):
            value = data[key]
            if type(value) == dict:
                del data[key]
                for _key in list(value.keys()):
                    data[key + '[' + _key + ']'] = value[_key]
            elif type(value) == list:
                del data[key]
                for _key in range(len(value)):
                    data[key + '[' + str(_key) + ']'] = value[_key]



import pymysql

from autoTestScheme.common import logger


class MySql(object):
    database_kwargs = {}

    def __init__(self):
        pass

    def excute(self, sql_str, is_excute=False, is_logger=True):
        """
        执行sql语句
        :param sql_str: 执行命令
        :param is_excute: 是否为执行语句
        :param is_logger: 是否打印日志
        :return:
        """
        result = []
        self.sql = pymysql.Connect(charset='utf8', **self.database_kwargs, read_timeout=300)
        cursor = self.sql.cursor()
        if is_logger is True:
            logger.debug("执行sql命令({}:{}):{}".format(self.database_kwargs['host'], self.database_kwargs['port'], sql_str))
        cursor.execute(sql_str)
        if is_excute is False:
            result = self.sql_fetch_json(cursor)
        else:
            self.sql.commit()
        cursor.close()
        if is_logger is True:
            logger.debug(result)
        self.sql.close()
        return result

    def select_database(self, db):
        self.sql.select_db(db)
        logger.debug("切换数据库:{}".format(db))

    def connect(self, database_kwargs):
        """
        连接数据库
        :param database_kwargs: 数据库参数，字典，包含但不限于 host，port，user，password，database
        :return:
        """
        self.database_kwargs = database_kwargs
        # self.sql = pymysql.Connect(charset='utf8', **self.database_kwargs, read_timeout=300)

    def close(self):
        """
        关闭连接
        :return:
        """
        # self.sql.close()
        ...

    def sql_fetch_json(self, cursor: pymysql.cursors.Cursor):
        """
            将cursors的执行结果转换为字典
        """
        keys = []
        for column in cursor.description:
            keys.append(column[0])
        key_number = len(keys)
        json_data = []
        for row in cursor.fetchall():
            item = dict()
            for q in range(key_number):
                item[keys[q]] = row[q]
            json_data.append(item)
        return json_data





# -*- coding: utf-8 -*-
#!/usr/bin/env python
__owner__ = "熊润"
__created_date__ = "2019/9/23"

from xml.dom import minidom

from . import logger

"""

Usage:
    配置读取

"""

import json
import os
import traceback


class Json(object):
    
    def __init__(self, path):
        if not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        if not os.path.exists(path):
            with open(path, 'w+')as f:f.write('{}')
        config_str = ''
        self._path = path
        try:
            with open(self._path, encoding='utf8') as f:
                config_str = f.read()
            self.json_str = json.loads(config_str)
        except Exception:
            logger.error('error json format,path:{0},error:{1}'.format(self._path, traceback.format_exc()))
            self.json_str = {}

    def get_json(self):
        '''
            获取json字符串
        '''
        config_content = None
        try:
            config_content = json.dumps(self.json_str, sort_keys=True, ensure_ascii=False,
										indent=4, separators=(',', ':'))
        except Exception:
            print(self.json_str)
            print(traceback.format_exc())
        return config_content
    
    def get_count(self):
        '''
            获取json的键值对的个数
        '''
        return len(self.get_keys())

    def get_items(self):
        '''
            获取json的键值对的个数
        '''
        return self.json_str.items()

    def get_key(self, key):
        '''
            获取key对应的value值
        '''
        return self.json_str[key]

    get = get_key

    def put_key(self, key, value):
        '''
            修改key
        '''
        self.json_str[key] = value
        self.save()

    put = put_key

    def remove_key(self, key):
        if key in list(self.json_str.keys()):
            del self.json_str[key]
            self.save()

    def save(self):
        '''
            保存修改至文件内获取
        '''
        with open(self._path, 'w+') as f:
            f.write(self.get_json())

    def get_keys(self):
        '''
            获取key列表
        '''
        return list(self.json_str.keys())

    def get_values(self):
        '''
            获取value列表
        '''
        return list(self.json_str.values())

    def get_object(self):
        return self.json_str

    def set_object(self, json_str):
        self.json_str = json_str
        self.save()


class AllureXml(object):

    def __init__(self):
        self.dom = minidom.getDOMImplementation().createDocument(None, 'environment', None)
        self.root = self.dom.documentElement

    def append_child(self, environment):
        for value in environment:
            tmp_element = self.dom.createElement('parameter')
            tmp_element1 = self.dom.createElement('key')
            tmp_element1.appendChild(self.dom.createTextNode(value[0]))
            tmp_element2 = self.dom.createElement('value')
            tmp_element2.appendChild(self.dom.createTextNode(str(value[1])))
            tmp_element.appendChild(tmp_element1)
            tmp_element.appendChild(tmp_element2)
            self.root.appendChild(tmp_element)

    def save_path(self, save_path):
        # 保存文件
        with open(save_path, 'w+', encoding='utf-8') as f:
            self.dom.writexml(f, addindent='\t', newl='\n', encoding='utf-8')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
__title__  = 
__Time__   = 2020/12/19 08:47
__Author__ = 熊润
"""
import os
import sys
import platform
import tempfile
import subprocess

ATS_BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
TMP_FOLDER = tempfile.mkdtemp()
BASE_DIR = os.path.dirname(os.path.abspath(sys.argv[0]))

# 测试报告目录
REPORT_DIR = os.path.join(TMP_FOLDER, 'allure-results')
TMP_REPORT_DIR = os.path.join(TMP_FOLDER, 'allure-tmp-result')
TMP_REPORTS_DIR = os.path.join(TMP_FOLDER, 'allure-tmp-results')
if not os.path.exists(TMP_REPORTS_DIR):
    os.makedirs(TMP_REPORTS_DIR)
RUN_TMP_DIR = os.path.join(TMP_FOLDER, 'run_tmp')

ADDRESS_PATH = os.path.join(ATS_BASE_DIR, 'common', 'configure', 'address.json')

ALLURE_NAME = 'allure'
if platform.system() == 'Windows':
    ALLURE_NAME += ".bat"
ALLURE_PATH = os.path.join(ATS_BASE_DIR, 'allure', 'bin', ALLURE_NAME)
if platform.system() != 'Windows':
    subprocess.call("chmod 777 {}".format(ALLURE_PATH), shell=True)

TMP_RESULT_FOLDER = os.path.join(TMP_FOLDER, 'tmp_result')

CONFIG_FOLDER = os.path.join(BASE_DIR, 'config')



import dynaconf
from dynaconf.vendor.toml.decoder import TomlDecodeError
from .common import constant, common
from .common import logger
from .configuration.wb import webSocket
from .configuration.sql import MySql
from .configuration.request import requestBase
from rediscluster import RedisCluster
from redis import StrictRedis
from autoTestScheme.configuration import robot
import os, copy


class BaseDynaconf(dynaconf.Dynaconf):
    _hook_list = {}
    _client_list = {}
    _first_register = False
    report_env = []
    default_settings = {'other_elements': [], 'test_tags': ['info'], 'test_case': 'all', 'is_debug': False, 'process_num': 1,
                        'tag_name_list': {'all': '其他'}, 'tag_env_list': {'all': 'all'}, 'is_locust': False}

    def __init__(self, *args, **kwargs):
        super(BaseDynaconf, self).__init__(*args, **kwargs)
        self.init_run_config()

    def init_run_config(self):
        if self.exists('run'):
            self.set('run', {})
        for key, value in self.default_settings.items():
            if key not in self.run:
                setattr(self.run, key, value)
        
        self.report_env.append(['环境:', self.get_env_name()])
        tag_name = ';'.join([self.run.tag_name_list[i] for i in self.run.test_tags])
        self.report_env.append(['标签:', tag_name])
        self.report_env.append(['执行用例:', self.run.test_case])
        self.report_env.append(['并行数:', self.run.process_num])

        self.register_hook('redis', self.get_redis)
        self.register_hook('request', self.get_session)
        self.register_hook('sql', self.get_sql)
        self.register_hook('feishu', self.get_feishu)
        self.register_hook('ws', self.get_websocket)

    def __getattr__(self, item: str):
        try:
            result = super().__getattr__(item)
        except TomlDecodeError as e:
            raise TypeError("toml文件格式错误，{}".format(e))
        except Exception as e:
            raise KeyError("配置文件内未定义{}".format(item))
        for name, func in self._hook_list.items():
            if item.startswith(name):
                if item not in self._client_list:
                    self._client_list[item] = func(item, result)
                return self._client_list[item]
        return result

    def register_hook(self, name, func):
        self._hook_list[name] = func

    def get_redis(self, item, config):
        new_config = copy.deepcopy(config)
        is_colony = new_config.get('is_colony', True)
        if 'is_colony' in new_config:
            del new_config['is_colony']
        obj = None
        if is_colony is True:
            if 'db' in new_config:
                del new_config['db']
            obj = RedisCluster(**new_config)
        else:
            obj = StrictRedis(**new_config)
        return obj

    def get_session(self, item, config) -> requestBase:
        request = requestBase()
        if config.get("base_url", None) is not None:
            request.base_url = config['base_url']
            request.kwargs = config
            for i in config.get('api', []):
                request.read_api_folder(*i)
        else:
            logger.error("{} 注册request失败, 配置信息:{}".format(item, config))
        logger.info('注册{}'.format(item))
        return request

    def get_sql(self, item, config) -> requestBase:
        sql = MySql()
        sql.connect(config)
        return sql

    def get_feishu(self, item, config):
        access_token = config.get('access_token')
        secret = config.get('secret')
        url = config.get('url', "https://open.feishu.cn")
        app_id = config.get('app_id')
        app_secret = config.get('app_secret')
        app_verification_token = config.get('app_verification_token')
        chat_id = config.get('chat_id')
        feishu = robot.Feishu(access_token, secret, app_id, app_secret, app_verification_token, chat_id, url)
        return feishu

    def get_websocket(self, item, config):
        ws = webSocket(config)
        for i in config.get('api', []):
            ws.read_api_folder(*i)
        return ws

    def get_env_name(self):
        current_env = self.current_env
        tag_name_list = self.run.get('tag_name_list', None)
        if tag_name_list is None or current_env not in tag_name_list:
            if self.exists('name'):
                return self.name
            else:
                return current_env
        else:
            return self.run.tag_name_list[current_env]


settings = BaseDynaconf(envvar_prefix=False, merge_enabled=True, environments=True, load_dotenv=True,
                        env_switcher="ENV", root_path=constant.CONFIG_FOLDER, includes=['*.toml'])



import gevent
import threading


class Singleton(object):
    _instance_lock = threading.Lock()
    is_start = False
    start = None

    def __init__(self):
        pass

    def __new__(cls, *args, **kwargs):
        if not hasattr(Singleton, "_instance"):
            with Singleton._instance_lock:
                if not hasattr(Singleton, "_instance"):
                    Singleton._instance = object.__new__(cls)
        return Singleton._instance

    def run(self):
        if self.is_start is False:
            self.start()
            self.is_start = True

    def set_start(self, start):
        if self.start is None:
            self.start = start


singleton = Singleton()


class Case(object):

    def __init__(self):
        self.base_url = None
        self.singleton = singleton
        self.singleton.set_start(self.on_locust_setup)

    def on_locust_setup(self) -> object:
        """
        前置条件，只会执行一次
        :return:
        """
        self.logger.error('执行用例前置条件')
        self.singleton.base_url = self.settings.base_url
        self.singleton.index = 0
        self.setup_class()

    def append_task(self, func, *args):
        gevent.spawn(func, *args)



from setuptools import setup, find_packages

install_requires = [
    "locust",
    "redis-py-cluster",
    'pluggy==0.13.1',
    "loguru",
    "dingtalkchatbot",
    "allure-pytest",
    "pytest-ordering",
    "pymysql",
    "json_tools",
    "pytest~=6.2.5",
    "pako~=0.3.1",
    "websocket-client",
    "Faker",
    "dynaconf",
]

packages = find_packages("src")

long_description = "1.支持飞书应用机器人配置"


setup(name='autoTestScheme',
      version='0.1.0.10',
      url='https://gitee.com/xiongrun/auto-test-scheme',
      author='wuxin',
      description='auto test scheme',
      long_description=long_description,
      long_description_content_type='text/markdown',
      author_email='xr18668178362@163.com',
      install_requires=install_requires,
      project_urls={'Bug Tracker': 'https://gitee.com/xiongrun/auto-test-scheme/issues'},
      package_dir={'': 'src'},
      packages=packages,
      include_package_data=True,
      entry_points={'pytest11': ['pytest_autoTestScheme = autoTestScheme']},
      package_data={
          'demo': ['demo/*'],
          'autoTestScheme': ['allure/*'],
      },
      )





from autoTestScheme import run


if __name__ == "__main__":
    run = run.Run()
    tags = {'info': '冒烟'}
    run.load_case('data')
    run.load_allure_tmp('allure-results')
    run.register_tags(tags)
    run.run(tags=['info'], process_num=1, is_debug=False, env='dev', case='all')



import grpc
from .api import Api


class Grpc(Api):

    def __init__(self, host):
        super().__init__()
        self.channel = grpc.insecure_channel(host)

    def get_grpc(self, api_id):
        pb2, pb2_grpc = grpc.protos_and_services(self.get_api(api_id))



